<template>
  <div class="context-menu" v-show="visible">
    <!-- mask -->
    <div
      class="mask"
      v-show="visible"
      @click="close"
      @contextmenu="close"
    ></div>
    <div class="contain">
      <ul class="main-ul">
        <li
          class="main-li"
          v-for="(item, idx) in menuList"
          :key="'main_' + idx"
        >
          <div
            :class="['title', {disabled: item.disabled}]"
            @click="itemClick(item)"
            @mouseenter="itemHover(item)"
          >
            {{ item.title }}
          </div>
          <ul
            class="sub-ul"
            v-show="item.visible && item.children && item.children.length"
          >
            <li
              :class="['sub-li', {disabled: subItem.disabled}]"
              v-for="(subItem, index) in item.children"
              :key="'sub_' + index"
              @click="itemClick(subItem)"
            >
              {{ subItem.title }}
            </li>
          </ul>
        </li>
      </ul>
    </div>
  </div>
</template>

<script>
export default {
  name: "ContextMenu",
  model: {
    prop: "visible",
    event: "change",
  },
  props: {
    visible: {
      type: Boolean,
      required: true,
      default: false,
    },
    disabledList: {
      type: Array,
      default() {
        return [];
      },
    },
  },
  data() {
    const menuList = [
      {
        id: 0,
        title: "打开文件",
        method: "openFile",
        disabled: false,
        visible: false,
      },
      {
        id: 1,
        title: "新建",
        disabled: false,
        visible: false,
        children: [
          {
            id: 11,
            title: "新建根文件",
            // method: "openFile",
            disabled: false,
          },
          {
            id: 12,
            title: "新建子文件",
            // method: "openFile",
            disabled: false,
          },
        ],
      },
      {
        id: 2,
        title: "添加",
        disabled: false,
        visible: false,
        children: [
          {
            id: 13,
            title: "指定目录下所有文件",
            disabled: false,
          },
          {
            id: 14,
            title: "单一文件",
            disabled: false,
          },
        ],
      },
      {
        id: 3,
        title: "重命名",
        method: "openFile",
        disabled: false,
        visible: false,
      },
      {
        id: 4,
        title: "删除",
        method: "openFile",
        disabled: false,
        visible: false,
      },
    ];
    return {
      menuList,
    };
  },
  components: {},
  watch: {
    visible(n) {
      if(n) {
        this.initData();
      }
    }
  },
  methods: {
    itemClick(obj) {
      // console.log(obj)
      if (obj.method && "function" == typeof this[obj.method] && !obj.disabled) {
        this.close();
        this[obj.method](obj);
      }
    },
    close() {
      this.invisibleSubMenu();
      this.$emit("change", false);
    },
    showChildren(obj) {
      // this.menuList.forEach((v) => {
      //   v.visible = false;
      // });
      obj.visible = true;
    },
    invisibleSubMenu() {
      this.menuList.forEach((v) => {
        v.visible = false;
      });
    },
    itemHover(obj) {
      if (obj.disabled) {
        return false;
      }
      this.invisibleSubMenu();
      this.$nextTick(() => {
        obj.visible = true;
      });
    },
    openFile() {
      // console.log("openfile");
      window.api.send("open-file-dialog");
    },
    initEvents() {
      window.api.receive("selected-file", (path) => {
        console.log(path.filePaths);
        // this.path = path.filePaths;
      });
      window.api.receive("selected-directory", (path) => {
        console.log(path.filePaths);
      });
    },
    initData() {
      this.menuList.forEach((v) => {
        v.visible = false;
        v.disabled = this.disabledList.some((val) => val == v.id);
        (v.children || []).forEach((item) => {
          item.disabled = this.disabledList.some((val) => val == item.id);
        });
      });
      console.log(this.menuList, this.disabledList, 9090)
    },
  },

  mounted() {
    this.initData();
    this.initEvents();
  },
};
</script>

<style lang="less" scoped>
.context-menu {
  position: absolute;
  background: #fff;
  border: 1px #ccc solid;
  border-radius: 4px;
  padding: 4px;
  .contain {
    position: relative;
    z-index: 100;
    .main-ul {
      display: flex;
      flex-direction: column;
      .main-li {
        position: relative;
        cursor: pointer;
        flex: 1;
        border: 1px #ccc solid;
        padding: 4px;
        &:hover {
          background: #f0f0f0;
        }
        & ~ .main-li {
          margin-top: 2px;
        }
      }
      .sub-ul {
        // display: none;
        display: flex;
        flex-direction: column;
        position: absolute;
        left: 100%;
        top: 0;
        transform: translateX(6px);
        min-width: 10em;
        background: #f0f0f0;
        padding: 4px;
        .sub-li {
          border-radius: 4px;
          border: 1px #333 solid;
          background: #d9dce2;
          &:hover {
            background: #169bd5;
          }
        }
      }
      .disabled.disabled.disabled {
        cursor: not-allowed;
        color: #aaa;
        background: #ccc;
      }
    }
  }
  .mask {
    position: fixed;
    left: 0;
    right: 0;
    top: 0;
    bottom: 0;
    background: transparent;
    z-index: 99;
  }
}
</style>